@@ -40,7 +40,6 @@ use base 'ORM::Db::DBI';
 ## (
 ##     host        => string,
 ##     database    => string,
-##     namespace   => string || undef,
 ##     user        => string,
 ##     password    => string,
 ##     pure_perl_driver => boolean,
@@ -52,16 +51,9 @@ sub new
     my %arg   = @_;
     my $self;
 
-    unless( $arg{pure_perl_driver} )
-    {
-        $arg{data_source} = "DBI:Pg:dbname='$arg{database}';host='$arg{host}';port=".($arg{port}||5432);
-    }
-
-    $arg{driver} = $arg{pure_perl_driver} ? 'PgPP' : 'Pg';
-    $self        = $class->SUPER::new( %arg );
-
+    $arg{driver}              = $arg{pure_perl_driver} ? 'PgPP' : 'Pg';
+    $self                     = $class->SUPER::new( %arg );
     $self->{pure_perl_driver} = $arg{pure_perl_driver};
-    $self->{namespace}        = $arg{namespace} || 'public';
 
     return $self;
 }
@@ -106,7 +98,37 @@ sub qf { $_[0]->qi( $_[1] ); }
 ## OBJECT METHODS
 ##
 
-sub _namespace        { $_[0]->{namespace}; }
+
+sub begin_transaction
+{
+    my $self  = shift;
+    my %arg   = @_;
+
+    $self->{ta} = 1;
+    $self->do( query=>"BEGIN", error=>$arg{error} );
+}
+
+sub commit_transaction
+{
+    my $self  = shift;
+    my %arg   = @_;
+
+    delete $self->{ta};
+    $self->do( query=>"COMMIT", error=>$arg{error} );
+}
+
+sub rollback_transaction
+{
+    my $self  = shift;
+    my %arg   = @_;
+
+    delete $self->{ta};
+    unless( $self->{lost_connection} )
+    {
+        $self->do( query=>"ROLLBACK", error=>$arg{error} );
+    }
+}
+
 sub _pure_perl_driver { $_[0]->{pure_perl_driver}; }
 
 ## use: $id = $db->insertid()
@@ -122,7 +144,7 @@ sub insertid
     }
     elsif( $self->_pure_perl_driver )
     {
-        $id = $self->_PgPP_last_insert_id
+        $id = _PgPP_last_insert_id
         (
             $self->_db_handler,
             $self->database,
@@ -167,12 +189,11 @@ sub table_struct
     ## Fetch table structure
 
     # WARNING! DBD::PgPP driver does not support the following
-    # so version detection probably should be rewriten to use
+    # so version detection should be rewriten to use
     # "SELECT version()" or something else.
 
-    my $version = $self->_db_handler->{pg_server_version} || 0;
-    my $old_ver = $version < 70300;
-    my $catalog = $old_ver ? '' : 'pg_catalog.';
+    my $old_ver = $self->_db_handler->{pg_server_version} < 70300;
+    my $catalog = $old_ver ? 'pg_catalog.' : '';
 
     #ORM::DbLog->write_to_stderr(1);
     $res = $self->select
@@ -204,7 +225,7 @@ sub table_struct
                 a.attnum >= 0
                 AND c.relkind IN (\'r\',\'v\')
                 AND c.relname = ' . $self->qc( $arg{table} ) . '
-                '.( $old_ver ? '' : 'AND n.nspname = '.$self->qc( $self->_namespace ) ).'
+                '.( $old_ver ? '' : 'AND n.nspname = '.$self->qc( $self->database ) ).'
             ORDER BY "Index"'
         )
     );
@@ -263,20 +284,16 @@ sub _lost_connection
     my $self = shift;
     my $err  = shift;
 
-    # mysql: defined $err && ( $err == 2006 || $err == 2013 );
-    warn "Don't know how to verify whether error was caused by connection abort!";
+    # mysql - defined $err && ( $err == 2006 || $err == 2013 );
     undef;
 }
 
-# PgSQL does not support FOR UPDATE together with SELECT DISTINCT
-sub _ta_select { ''; }
-
 sub _parse_default_value
 {
     my $self  = shift;
     my $value = shift;
 
-    if( defined $value && $value =~ /^'(.*)'::[^:]+$/ )
+    if( $value =~ /^'(.*)'::[^:]+$/ )
     {
         $value = $1;
         $value =~ s/''/'/g;
@@ -289,10 +306,10 @@ sub _parse_default_value
     return $value;
 }
 
-# Cloned from DBD::Pg
+# Copied from DBD::Pg
 sub _PgPP_last_insert_id
 {
-    my ($self, $dbh, $catalog, $schema, $table, $col, $attr) = @_;
+    my ($dbh, $catalog, $schema, $table, $col, $attr) = @_;
 
     ## Our ultimate goal is to get a sequence
     my ($sth, $count, $SQL, $sequence);
@@ -305,10 +322,6 @@ sub _PgPP_last_insert_id
     $table = '' if ! defined $table;
     my $cachename = "lii$table$schema";
 
-    my $version = $self->_db_handler->{pg_server_version} || 0;
-    my $old_ver = $version < 70300;
-    my $use_cat = $old_ver ? '' : 'pg_catalog.';
-
     if (defined $attr and length $attr) {
         ## If not a hash, assume it is a sequence name
         if (! ref $attr) {
@@ -337,7 +350,7 @@ sub _PgPP_last_insert_id
         my @args = ($table);
 
         ## Only 7.3 and up can use schemas
-        $schema = '' if( $old_ver );
+        $schema = '' if $dbh->{private_dbdpg}{version} < 70300;
 
         ## Make sure the table in question exists and grab its oid
         my ($schemajoin,$schemawhere) = ('','');
@@ -346,7 +359,7 @@ sub _PgPP_last_insert_id
             $schemawhere = "\n AND n.nspname = ?";
             push @args, $schema;
         }
-        $SQL = "SELECT c.oid FROM ${use_cat}pg_class c $schemajoin\n WHERE relname = ?$schemawhere";
+        $SQL = "SELECT c.oid FROM ${DBD::Pg::dr::CATALOG}pg_class c $schemajoin\n WHERE relname = ?$schemawhere";
         $sth = $dbh->prepare($SQL);
         $count = $sth->execute(@args);
         if (!defined $count or $count eq '0E0') {
@@ -358,7 +371,7 @@ sub _PgPP_last_insert_id
         my $oid = $sth->fetchall_arrayref()->[0][0];
         ## This table has a primary key. Is there a sequence associated with it via a unique, indexed column?
         $SQL = "SELECT a.attname, i.indisprimary, substring(d.adsrc for 128) AS def\n".
-            "FROM ${use_cat}pg_index i, ${use_cat}pg_attribute a, ${use_cat}pg_attrdef d\n ".
+            "FROM ${DBD::Pg::dr::CATALOG}pg_index i, ${DBD::Pg::dr::CATALOG}pg_attribute a, ${DBD::Pg::dr::CATALOG}pg_attrdef d\n ".
                 "WHERE i.indrelid = $oid AND d.adrelid=a.attrelid AND d.adnum=a.attnum\n".
                     "  AND a.attrelid=$oid AND i.indisunique IS TRUE\n".
                         "  AND a.atthasdef IS TRUE AND i.indkey[0]=a.attnum\n".
